//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MindFusion.Gauges;


namespace MindFusion.Gauges.WinForms.Samples.CS.Equalizer
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            MindFusion.Gauges.WinForms.LinearGauge[] verticalGauges =
				new MindFusion.Gauges.WinForms.LinearGauge[8];
            string[] labels = new string[] { "31 Hz", "62 Hz", "125 Hz", "250 Hz", "500 Hz", "1 kHz", "2 kHz", "4 kHz" };

            MajorTickSettings mj;
            MiddleTickSettings md;
            MinorTickSettings mn;
            Pointer p;
            
            int w = (Width - 2 * 15) / 8;
            for (int i = 0; i < 8; ++i)
            {
				MindFusion.Gauges.WinForms.LinearGauge g =
					new MindFusion.Gauges.WinForms.LinearGauge();
                verticalGauges[i] = g;
                p = new Pointer();
                p.Shape = PointerShape.Custom;
                p.IsInteractive = true;
                p.PointerWidth = new Length(24, LengthType.Relative);
                p.PointerHeight = new Length(48, LengthType.Relative);

                g.PaintPointer += g_PaintPointerVertical;
                LinearScale sc = new LinearScale();
                sc.Pointers.Add(p);
                sc.Left = new Length(50, LengthType.Relative);
                sc.Top = new Length(10, LengthType.Relative);
                sc.ScaleLength = new Length(80, LengthType.Relative);
                sc.StartWidth = sc.EndWidth = new Length(20, LengthType.Relative);
                sc.MinValue = -24;
                sc.MaxValue = 24;
                // make scale transparent instead of cancel in prepaint, to use default pointer and tick painting
                sc.Stroke = new MindFusion.Drawing.Pen(Color.Transparent);
                sc.Fill = new MindFusion.Drawing.SolidBrush(Color.Transparent);
                g.Scales.Add(sc);

                g.Width = w;
                g.Height = 400;
                g.Left = verticalGauges[0].Left + w * i;
                g.PrepaintBackground += CancelPaint;
                g.PrepaintForeground += CancelPaint;
                g.PrepaintScale += g_PrepaintScale;
                g.Orientation = MindFusion.Gauges.Orientation.Vertical;

                mj = sc.MajorTickSettings;
                mj.Count = 2;
                mj.TickShape = TickShape.Rectangle;
                mj.TickWidth = new Length(2, LengthType.Relative);
                mj.TickHeight = new Length(0.5f, LengthType.Relative);
                mj.TickAlignment = Alignment.InnerInside;
                mj.LabelAlignment = Alignment.InnerCenter;
                mj.LabelOffset = new Length(25, LengthType.Relative);
                mj.TickOffset = new Length(1);
                mj.FontSize = new Length(20, LengthType.Relative);
                mj.LabelRotation = LabelRotation.BaselineToCenter;
                mj.LabelFormat = "0.# dB";

                md = sc.MiddleTickSettings;
                md.Count = 10;
                md.TickShape = TickShape.Rectangle;
                md.TickWidth = new Length(1.5f, LengthType.Relative);
                md.TickHeight = new Length(1);
                md.TickOffset = new Length(1);
                md.TickAlignment = Alignment.InnerInside;
                md.ShowLabels = false;

                sc.MinorTickSettings.ShowLabels = sc.MinorTickSettings.ShowTicks = false;

                Controls.Add(g);

                Label lb = new Label();
                lb.Text = labels[i];
                lb.Top = g.Bottom + 1;
                lb.Left = g.Left + g.Width /2 - lb.PreferredWidth /2;
                lb.Width = w;
                lb.Font = new Font(Font, FontStyle.Bold);
                Controls.Add(lb);
            }

            OvalGauge og = null;
            OvalScale os;

            for (int i = 0; i < 4; ++i)
            {
                og = new OvalGauge();
                og.Width = og.Height = verticalGauges[0].Height/3;
                og.Top = 430;
                og.Left = verticalGauges[0].Left + i * (og.Width + 20);
                og.PrepaintBackground += CancelPaint;
                og.PrepaintForeground += CancelPaint;
                og.PrepaintPointer += g_PrepaintPointerOval;

                os = new OvalScale();
                
                p = new Pointer();
                p.IsInteractive = true;
                p.PointerWidth = p.PointerHeight = new Length(165, LengthType.Relative);
                
                os.Pointers.Add(p);
                os.Stroke = new MindFusion.Drawing.Pen(Color.Transparent);
                os.Fill = new MindFusion.Drawing.SolidBrush(Color.Transparent);
                os.MinValue = -50;
                os.MaxValue = 50;
                os.ScaleRelativeRadius = 0.45f;
                og.Scales.Add(os);

                mj = os.MajorTickSettings;
                mj.Count = 5;
                mj.LabelAlignment = Alignment.OuterCenter;
                mj.LabelRotation = LabelRotation.None;
                mj.LabelOffset = new Length(-15, LengthType.Relative);
                mj.FontSize = new Length(20, LengthType.Relative);

                md = os.MiddleTickSettings;
                md.ShowLabels = false;

                mn = os.MinorTickSettings;

                mn.ShowTicks = true;
                mn.TickShape = md.TickShape = mj.TickShape = TickShape.Rectangle;
                mn.Fill = md.Fill = mj.Fill = new MindFusion.Drawing.SolidBrush(Color.Black);
                mn.Stroke = md.Stroke = mj.Stroke = new MindFusion.Drawing.Pen(Color.Black);
                mn.TickWidth = md.TickWidth = mj.TickWidth = new Length(12, LengthType.Relative);
                mn.TickHeight = md.TickHeight = mj.TickHeight = new Length(4, LengthType.Relative);

                mj.Count = 5;
                mj.TickShape = TickShape.Rectangle;
                mj.Fill = new MindFusion.Drawing.SolidBrush(Color.Black);
                mj.Stroke = new MindFusion.Drawing.Pen(Color.Black);
                mj.TickWidth = new Length(12, LengthType.Relative);
                mj.TickHeight = new Length(4, LengthType.Relative);
                mj.LabelAlignment = Alignment.OuterCenter;
                mj.LabelRotation = LabelRotation.None;
                mj.LabelOffset = new Length(-15, LengthType.Relative);
                mj.FontSize = new Length(20, LengthType.Relative);

                Controls.Add(og);
            }

            //on/off gauge
            OvalGauge onoff = new OvalGauge();
            onoff.Size = og.Size;
            onoff.Left = og.Right + 20;
            onoff.Top = og.Top;
            onoff.PrepaintBackground += CancelPaint;
            onoff.PrepaintForeground += CancelPaint;

            //onoff.PaintScale += new EventHandler<CustomPaintEventArgs>(onoff_PaintScale);
            os = new OvalScale();
            os.StartAngle = 225;
            os.EndAngle = 315;
            os.MinValue = 0;
            os.MaxValue = 1;
            os.Stroke = new MindFusion.Drawing.Pen(Color.Transparent);
            os.Fill = new MindFusion.Drawing.SolidBrush(Color.Transparent);
            os.QueryLabelValue += new EventHandler<QueryLabelValueEventArgs>(os_QueryLabelValueOval);

            mj = os.MajorTickSettings;
            mj.Count = 1;
            mj.TickShape = TickShape.Rectangle;
            mj.TickWidth = new Length(15, LengthType.Relative);
            mj.TickHeight = new Length(4, LengthType.Relative);
            mj.Fill = new MindFusion.Drawing.SolidBrush(Color.Black);
            mj.FontSize = new Length(20, LengthType.Relative);
            mj.LabelAlignment = Alignment.OuterOutside;
            mj.LabelRotation = LabelRotation.None;
            mj.LabelOffset = new Length(-10, LengthType.Relative);

            os.MiddleTickSettings.ShowTicks = os.MiddleTickSettings.ShowLabels = false;

            p = new Pointer();
            p.PointerWidth = p.PointerHeight = new Length(140, LengthType.Relative);
            p.IsDiscrete = p.IsInteractive = true;
            onoff.PrepaintPointer += new EventHandler<PrepaintEventArgs>(onoff_PrepaintPointer);

            os.Pointers.Add(p);
            
            onoff.Scales.Add(os);
            Controls.Add(onoff);
        }

        void os_QueryLabelValueOval(object sender, QueryLabelValueEventArgs e)
        {
            if (e.CalculatedLabelValue == 0)
                e.NewValue = "Off";
            else if (e.CalculatedLabelValue == 1)
                e.NewValue = "On";
            else
                e.NewValue = "";
        }

        void onoff_PrepaintPointer(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;
            // button profile form _/-\_
            object[] blend = { 0f, Utils.Gray(0xA0), 0.4f, Utils.Gray(0xD0), //lit half, _
                0.4f, Utils.Gray(0xFF), 0.41f, Utils.Gray(0xFF), //lit edge, /
                0.41f, Utils.Gray(0xA1), 0.6f, Utils.Gray(0xA1), //plateau, -
                0.6f, Utils.Gray(0x80), 0.8f, Utils.Gray(0xA0) // dim half, _
            };
            using (Brush br = Utils.LinearGradient(90, blend).CreateGdiBrush(Utils.ToRectangleF(e.Element.RenderSize)))
                e.Graphics.FillEllipse(br, Utils.ToRectangleF(e.Element.RenderSize));
            using (Pen p = new MindFusion.Drawing.Pen(Color.FromArgb(0xFF, 0x5f, 0x5f, 0x5f)).CreateGdiPen())
                e.Graphics.DrawEllipse(p, Utils.ToRectangleF(e.Element.RenderSize));
        }

        void g_PrepaintPointerOval(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;

            Ellipse outer = new Ellipse();
            outer.Fill = new MindFusion.Drawing.LinearGradientBrush(Color.FromArgb(0xA0, 0xC0, 0xC0), Color.FromArgb(0x10, 0x30, 0x30));
            e.PaintVisualElement(outer, e.Element.RenderSize);

            Ellipse inner1 = new Ellipse();
            inner1.Fill = new MindFusion.Drawing.SolidBrush(Color.FromArgb(0x60, 0x80, 0x80));
            inner1.Margin = new MindFusion.Thickness(0.15f);
            e.PaintVisualElement(inner1, e.Element.RenderSize);

            Ellipse inner2 = new Ellipse();
            inner2.Fill = new MindFusion.Drawing.SolidBrush(Color.FromArgb(0x40, 0x60, 0x60));
            inner2.Margin = new MindFusion.Thickness(0.2f);
            e.PaintVisualElement(inner2, e.Element.RenderSize);

            SizeF sz = e.Element.RenderSize;
            using (Pen p = new Pen(Color.Black))
                e.Graphics.DrawLine(p, new PointF(sz.Width /2 , sz.Height / 2), new PointF(sz.Width * 0.9f, sz.Height / 2));
            
            Ellipse dot = new Ellipse();
            dot.Fill = new MindFusion.Drawing.SolidBrush(Color.Red);
            dot.Margin = new MindFusion.Thickness(0.9f, 0.45f, 0, 0.45f);
            e.PaintVisualElement(dot, e.Element.RenderSize);
        }

        void CancelPaint(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;
        }

        void g_PrepaintScale(object sender, PrepaintEventArgs e)
        {
            RoundRectangle r = new RoundRectangle();
            r.Roundness = 20;
            r.Margin = new MindFusion.Thickness(0.38f, 0.055f, 0.38f, 0.05f, true);
            r.Fill = Utils.LinearGradient(0, 0f, Utils.Gray(0x66), 0.2f, Utils.Gray(0xCC), 0.4f, Utils.Gray(0xCC), 1f, Utils.Gray(0x88));
            e.PaintVisualElement(r, e.Element.RenderSize);

            RectangleF filledBounds = r.Margin.ToAbsolute(e.Element.RenderSize).Apply(Utils.ToRectangleF(e.Element.RenderSize));
            float mid = e.Element.RenderSize.Width / 2;
            using (Pen p = new Pen(Color.Black))
                e.Graphics.DrawLine(p, mid - 0.5f, filledBounds.Top + 11, mid - 0.5f, filledBounds.Bottom - 11);
            using (Pen p = new Pen(Color.White))
                e.Graphics.DrawLine(p, mid + 0.5f, filledBounds.Top + 11, mid + 0.5f, filledBounds.Bottom - 11);
        }

        void g_PaintPointerVertical(object sender, CustomPaintEventArgs e)
        {
            RectangleF bounds = Utils.ToRectangleF(e.Element.RenderSize);
            RoundRectangle r = new MindFusion.Gauges.RoundRectangle();
            RoundRectangle r2 = new MindFusion.Gauges.RoundRectangle();
            r.Roundness = r2.Roundness = 20;
            r.Stroke = new MindFusion.Drawing.Pen(Color.Black);

            e.PaintVisualElement(r, bounds.Size);

            r2.Stroke = new MindFusion.Drawing.Pen(Color.White);
            r2.X = r2.Y = 1;
            r2.Fill = new MindFusion.Drawing.LinearGradientBrush(Color.White, Color.FromArgb(0x66, 0x66, 0x66));
            bounds.Inflate(-1, -1);
            e.PaintVisualElement(r2, bounds.Size);

            float middle = e.Element.RenderSize.Height / 2;
            using (Pen p = new Pen(Color.White))
                e.Graphics.DrawLines(p, new PointF[] { new PointF(3, middle - 0.5f), new PointF(e.Element.RenderSize.Width - 3, middle -0.5f) });

            using (Pen p = new Pen(Color.DimGray))
                e.Graphics.DrawLines(p, new PointF[] { new PointF(3, middle + 0.5f), new PointF(e.Element.RenderSize.Width - 3, middle + 0.5f) });
        }
    }
}